'BootLoader.vbs
Option Explicit

dim i

' 0xffe04010
dim dl_address_hi : dl_address_hi=65504
dim dl_address_lo : dl_address_lo=16400

'Check arguments
if Wscript.Arguments.Count <> 1 then
    MsgBox "Incorrect number of arguments passed" & VbCrLf & VbCrLf & "Usage:" & VbCrLf & "  BootLoader.Vbs filename" & VbCrLf & "    where filename is the accesspath and name of the file to download." & VbCrLf & "    Example: BootLoader.vbs myapp.bin", vbCritical, "BootLoader.vbs"
    Wscript.quit(21924)
end if

'Check if file passed as argument exists
dim bootfile  : bootfile  = Wscript.arguments(0)
dim oFs        : set oFs        = CreateObject("Scripting.FileSystemObject")
On Error Resume Next
if  NOT(oFs.FileExists(bootfile)) then
    MsgBox "File (" & bootfile & ") not found", vbCritical, "BootLoader.Vbs"
    Wscript.quit 999
end if

Dim bfile : Set bfile = oFS.GetFile(BootFile)
Dim filelength : filelength = bfile.size
Dim numbytes : numbytes = 0

'Create EPObject
Err.Clear
dim oEP        : set oEP        = CreateObject("EPObject.EmbPipe")
if Err.number <> 0 then
    MsgBox "Embedded Pipe object can not be created", vbCritical, "BootLoader.Vbs"
    Wscript.quit 999
end if

'Set EP parameters
oEP.TargetDev = &Hc1cababa      ' Vendor and product ID of target device
oEP.CtrlOrBulk = 0              ' use ctrl transfers
oEP.ClassOrVendor = 1           ' ctrl transfers are vendor specific
oEP.Receipent = 3               ' target of ctrl transfers is "Other"
oEP.InternalBufLength = 1024    ' max transfer size. Shall not be bigger than
                                ' buffer size on the enbedded side.

''''''''''''''''''''''''''''''''''''''''''
' 0. Open embedded pipe:
'    Try to open it for 10 seconds

for i=1 to 100
  Err.Clear
  oEP.Open()
  if Err.number = 0 then exit for 
  oEP.Close()
  wscript.sleep 100
next


if Err.number <> 0 then
    MsgBox "Target device can not be opened" & VbCrLf & "Error code: " & CStr(Err.Number) & VbCrLf & "Description: " & Err.Description, vbCritical, "BootLoader.Vbs"
        'MsgBox ("DEBUG: Error code: " & CStr(Err.Number) & " Description:" & Err.Description)
    Wscript.quit 999
end if

''''''''''''''''''''''''''''''''''''''''''
' 1.1 send reset command. Target device will
'    reset itself. If the device can not controll
'    the USB pull-up resistor, then it needs
'    to  be re-enumerated to be able to talk on
'    USB again.
oEP.Cmd_Out = &H2F      ' CMD_RESET
oEP.urbIndex = 0        ' Not used by Reset
oEP.urbValue = 0        ' Not used by Reset
Err.Clear
oEP.Write
if Err.number <> 0 then
    MsgBox "Reset command failed" & VbCrLf & "Error code: " & CStr(Err.Number) & VbCrLf & "Description: " & Err.Description, vbCritical, "BootLoader.Vbs"
    oEP.Close
    Wscript.quit 999
end if

'
' 1.2 reset USB port to get device re-enumerated.
'     This is a soft reset, and thus the EP driver
'     remains the driver of the target device.
Dim rt 'for object EPOBJECTLib.ResetTypeEnum
rt = 1  'rt_port_soft
Err.Clear
Call oEP.Reset(rt)

if Err.number <> 0 then
    MsgBox "USB port reset failed" & VbCrLf & "Error code: " & CStr(Err.Number) & VbCrLf & "Description: " & Err.Description, vbCritical, "BootLoader.Vbs"
    oEP.Close
    Wscript.quit 999
end if

''''''''''''''''''''''''''''''''''''''''''
' 2. Erase flash        
oEP.Cmd_Out = &H10            ' CMD_ERASE 0x10
oEP.urbIndex = dl_address_hi   ' HIWORD of start address
oEP.urbValue = dl_address_lo   ' LOWORD of start address
Err.Clear
Call oEP.Write(FileLength,4,numbytes)
if Err.number <> 0 Or numbytes <> 4 then
    MsgBox "Flash erase failed" & VbCrLf & "Error code: " & CStr(Err.Number) & VbCrLf & "Description: " & Err.Description, vbCritical, "BootLoader.Vbs"
    oEP.Close
    Wscript.quit 999
end if

''''''''''''''''''''''''''''''''''''''''''
' 3. Download data
oEP.Cmd_Out = &H20        ' CMD_DWLOAD  0x20
oEP.urbIndex = dl_address_hi   ' HIWORD of start address
oEP.urbValue = dl_address_lo   ' LOWORD of start address
Err.Clear
Call oEP.SendFile(bfile.Path,numbytes)
if Err.number <> 0 Or numbytes <> filelength then
    MsgBox "Error during download" & VbCrLf & "Error code: " & CStr(Err.Number) & VbCrLf & "Description: " & Err.Description, vbCritical, "BootLoader.Vbs"
    oEP.Close
    Wscript.quit 999
end if

''''''''''''''''''''''''''''''''''''''''''
' 4.1 Let targed device start application.
'     If the application wants to use the
'     USB, then the device shall be
'     re-enumerated to get USB working.
oEP.Cmd_Out = &H40        ' CMD_START 0x40
oEP.urbIndex = 0        ' Not used by start
oEP.urbValue = 0        ' Not used by start
Err.Clear
oEP.WriteCmd
if Err.number <> 0 then
    MsgBox "Start command failed" & VbCrLf & "Error code: " & CStr(Err.Number) & VbCrLf & "Description: " & Err.Description, vbCritical, "BootLoader.Vbs"
    oEP.Close
    Wscript.quit 999
end if

'
' 4.2 Reset USB port to get device re-enumerated.
'     This is a hard reset, and thus windows will
'     try to find a new driver for the device.
rt = 0  'rt_port_hard
Err.Clear
Call oEP.Reset(rt)

''''''''''''''''''''''''''''''''''''''''''
' 5. close Embedded pipe
'
oEP.Close
if Err.number <> 0 then
    MsgBox "Error Closing Embedded pipe!" & VbCrLf & "Error code: " & CStr(Err.Number) & " Description:" & Err.Description, vbCritical, "BootLoader.Vbs"
    Wscript.quit 999
end if

MsgBox "Download Successful!", vbInformation, "BootLoader.Vbs"
Wscript.quit 0

